"""
The PMT model.

# Arguments
- `photocathode_area`: the total are of the photocathode [m^2]
- `quantum_efficiency`: a callable which returns the quantum efficiency
  for a given wavelength [ns]
- `angular_acceptance`: a callable which returns the angular acceptance
  (for values between -1 and 1) for a given wavelength [ns]
"""
struct PMTModel{T1,T2}
    photocathode_area::Float64
    quantum_efficiency::T1
    angular_acceptance::T2
end

const KM3NeTPMT = PMTModel(
    45.4e-4,
    # The quantum efficiency includes absorption in glass and gel.
    LinearInterpolator(
        [
            0,
            270,
            275,
            280,
            285,
            290,
            295,
            300,
            305,
            310,
            315,
            320,
            325,
            330,
            335,
            340,
            345,
            350,
            355,
            360,
            365,
            370,
            375,
            380,
            385,
            390,
            395,
            400,
            405,
            410,
            415,
            420,
            425,
            430,
            435,
            440,
            445,
            450,
            455,
            460,
            465,
            470,
            475,
            480,
            485,
            490,
            495,
            500,
            505,
            510,
            515,
            520,
            525,
            530,
            535,
            540,
            545,
            550,
            555,
            560,
            565,
            570,
            575,
            580,
            585,
            590,
            595,
            600,
            605,
            610,
            615,
            620,
            625,
            630,
            635,
            640,
            645,
            650,
            655,
            660,
            665,
            670,
            675,
            680,
            685,
            690,
            695,
            700,
            705,
            710,
            999999,
        ],

        # collection efficiency (with factor 0.9) correction
        0.01 *
        0.9 *
        [
            0.0,
            0.0,
            0.0,
            0.0,
            0.0,
            0.0,
            0.1,
            0.1,
            0.9,
            1.8,
            4.6,
            7.4,
            11.3,
            15.2,
            18.1,
            21.0,
            22.4,
            23.7,
            24.5,
            25.3,
            25.8,
            26.3,
            26.2,
            26.0,
            26.6,
            27.1,
            26.9,
            26.8,
            26.5,
            26.3,
            26.0,
            25.7,
            25.6,
            25.4,
            25.1,
            24.9,
            24.5,
            24.1,
            23.5,
            22.9,
            22.1,
            21.3,
            20.6,
            19.9,
            19.5,
            19.1,
            18.8,
            18.5,
            18.1,
            17.7,
            16.8,
            15.9,
            14.3,
            12.7,
            11.5,
            10.2,
            9.5,
            8.7,
            8.1,
            7.6,
            7.2,
            6.8,
            6.4,
            6.0,
            5.6,
            5.1,
            4.8,
            4.4,
            4.0,
            3.6,
            3.3,
            3.0,
            2.7,
            2.3,
            2.1,
            1.9,
            1.7,
            1.5,
            1.4,
            1.2,
            1.0,
            0.9,
            0.7,
            0.6,
            0.5,
            0.4,
            0.3,
            0.2,
            0.1,
            0.0,
            0.0,
        ],
        NoBoundaries(),
    ),

    # using eps(), max angle and two zero values in the second vector to
    # get exact zeros outside of the upper boundary defined in Jpp (i.e. >=0.4)
    LinearInterpolator(
        [
            -1.00,
            -0.95,
            -0.90,
            -0.85,
            -0.80,
            -0.75,
            -0.70,
            -0.65,
            -0.60,
            -0.55,
            -0.50,
            -0.45,
            -0.40,
            -0.35,
            -0.30,
            -0.25,
            -0.20,
            -0.15,
            -0.10,
            -0.05,
            0.00,
            0.05,
            0.10,
            0.15,
            0.20,
            0.25,
            0.30,
            0.35,
            0.40,
            0.40 + eps(),
            1.0,
        ],
        [
            1.621,
            1.346,
            1.193,
            1.073,
            0.973,
            0.877,
            0.790,
            0.711,
            0.640,
            0.575,
            0.517,
            0.450,
            0.396,
            0.341,
            0.295,
            0.249,
            0.207,
            0.166,
            0.128,
            0.095,
            0.065,
            0.038,
            0.017,
            0.006,
            0.003,
            0.002,
            0.001,
            0.001,
            0.001,
            0.000,
            0.000,
        ],
        NoBoundaries(),
    ),
)
